﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;


namespace PojeClient {
    // 棋譜のデータ保存と入出力を行うクラス
    class SGFManager {
        private String fPlayerName1, fPlayerName2;
        private int fLimitTime;
        private String fFileName;
        
        private int fLastTurn;
        private String[] fMoves; 
        // １つのターンは B[xrcc]W[xrcc]; の形をしている
        // 横x=1..7 回転r={R,U,L,D} 色c=1..4  連鎖中などはB[] 終了後にnextを見せるための B[cc] 形式もある
        private List<int> fColors1, fColors2; // nextcolorsのストック

        public void setLimitTime(int t) {
            fLimitTime = t;
        }

        public void setPlayerNames(String name1, String name2) {
            fPlayerName1 = name1;
            fPlayerName2 = name2;
        }

        public void setFileName(String filename) {
            fFileName = filename;
        }

        public void initialize(int maxTurn) {
            fMoves = new String[maxTurn + 10];
            for (int t = 0; t < fMoves.Length; t++) fMoves[t] = "";
        }

        public void setActions(int turn, Action a1, Action a2) {
            String s1 = "B[____]";
            if (a1 != null) {
                s1 = "B[" + a1.getX().ToString() + Settings.RotateChars[a1.getR()] + a1.getColor(0).ToString() + a1.getColor(1).ToString() + "]";
            }
            String s2 = "W[____]";
            if (a2 != null) {
                s2 = "W[" + a2.getX().ToString() + Settings.RotateChars[a2.getR()] + a2.getColor(0).ToString() + a2.getColor(1).ToString() + "]";
            }
            fMoves[turn] = s1 + s2;
        }

        // 終了処理．残っている（見えている）nextcolor を書き足す．0が入っている場合（next生成前）は無視．
        public void endGame(int turn, int[] next1, int[] next2) {
            fLastTurn = turn;
            for (int addc = 0; addc < next1.Length / 2; addc++) {
                if (next1[addc * 2] > 0) {
                    fMoves[turn + addc] = "B[__" + next1[addc * 2].ToString() + next1[addc * 2 + 1].ToString() + "]"
                                        + "W[__" + next2[addc * 2].ToString() + next2[addc * 2 + 1].ToString() + "]";
                }
            }
        }

        // 保持している内容で sgfファイルを吐きだす
        public void saveToFile() {
            using (StreamWriter sw = new StreamWriter(fFileName)) {
                sw.WriteLine("(;FF[4]CA[UTF-8]GM[-1]SZ[" + Settings.BoardWidth + "x" + Settings.BoardHeight + "]TM["+fLimitTime+"]");
                sw.WriteLine("RU[c=" + Settings.NumColor + ",v=" + Settings.NumVanish + ",x=" + Settings.NextOpenTurns + ",default]");
                sw.WriteLine("PB[" + fPlayerName1 + "]PW[" + fPlayerName2 + "];");

                for (int t = 1; t < fMoves.Length; t++) {
                    if (fMoves[t].Equals("") == false) {
                        sw.WriteLine(fMoves[t] + ";");
                    }
                }
                sw.WriteLine(")");
            }
        }

        // sgfファイルの読み込みと分析格納
        // とりあえずヘッダ部分は無視する
        // 解釈できなかったら return false　読み込み時に事前チェックしておくほうが本当は良い
        public bool loadFromFile(String fileName) {
            fFileName = fileName;

            // まず一旦全部読む
            String fullLine = "";
            using (StreamReader sr = new StreamReader(fileName)) {
                String line;
                while ((line = sr.ReadLine()) != null) {
                    fullLine += line;
                }
            }

            // セミコロンで分解
            String[] lines = fullLine.Split(';');

            // 1つ目はカッコ
            // 2つ目の要素は設定など
            Logger.log("Loading " + fileName);
            Logger.log("  gamedata : " + lines[1]);

            fColors1 = new List<int>();
            fColors2 = new List<int>();
            fMoves = new String[lines.Length+10];
            
            // 3つめの要素～最後一個手前までが本体
            // 読み込みながら fColors のストックを足していく
            for (int i = 2; i < lines.Length - 1; i++) {
                fMoves[i - 1] = lines[i];
                String s1 = lines[i].Substring(4,1);
                String s2 = lines[i].Substring(5,1);
                if (s1.Equals("_") == false) {
                    fColors1.Add(Int32.Parse(s1));
                    fColors1.Add(Int32.Parse(s2));
                }
                String s3 = lines[i].Substring(11,1);
                String s4 = lines[i].Substring(12,1);
                if (s3.Equals("_") == false) {
                    fColors2.Add(Int32.Parse(s3));
                    fColors2.Add(Int32.Parse(s4));
                }
            }

            return true;
        }
                
        // 配珠色を先頭から１つ抜く
        // 色が必要になるタイミングは，実際に着手されるのより前になる（しかも連鎖とかあると離れる）のでややこしく，Listにする
        public int pickColors(int playerIdx) {
            if (playerIdx == 0) {
                int c = fColors1[0];
                fColors1.RemoveAt(0);
                return c;
            } else {
                int c = fColors2[0];
                fColors2.RemoveAt(0);
                return c;
            }
        }

        // 行動の x, rを返す
        // 行動が必要なところ（連鎖中以外）でこれが呼ばれて，なお "__" なら，そこで終了したということ
        public Point getActionAsPoint(int turn, int playerIdx) {
            String sx = fMoves[turn].Substring(playerIdx * 7 + 2, 1);
            if (sx.Equals("_")) {
                return new Point(-1,-1);
            }
            int x = Int32.Parse(sx);

            String sr = fMoves[turn].Substring(playerIdx * 7 + 3, 1);
            int r = -1;
            for (int i = 0; i < 4; i++) {
                if (sr.Equals(Settings.RotateChars[i]) == true) r = i;
            }
            if (r == -1) Logger.show("SGF Error: turn " + turn);
            return new Point(x, r);
        }




















    }
}
